/*
     tplib.h 

     Copyright (C) 2002 Christian Troger <ct@aim-sw.com>

     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
     as published by the Free Software Foundation; either version 2
     of the License, or (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __TPLIB
#define __TPLIB

#define TPLIB_OK		0
#define TPLIB_ERR_DEVICE	-1

typedef struct {
  unsigned char L_Service;
  unsigned char priority;
  unsigned char repeated;
  unsigned short source_addr;
  unsigned short dest_addr;
  unsigned char datalength; /* only size of data =L2_Length+2 */
  unsigned char data[64];
} L_Data_Frame;

int tp_eib_write_telegram (int device, L_Data_Frame ld);
int tp_eib_extract_L_Data_Frame  (unsigned short length, unsigned char data[64], L_Data_Frame *ld);

/* EIB */

/* Data Link Layer */

#define L_DATA_REQ			0x90
#define L_DATA_REQ_LONG			0x10
#define L_POLL_DATA_REQ			0xF0
#define L_DATA_CONF			0x8B
#define L_DATA_NO_CONF			0x0B
#define L_ACKFRAME_ACK			0xCC
#define L_ACKFRAME_NAK			0x0C
#define L_ACKFRAME_BUSY			0xC0

#define L_NREPEAT_DATA			0x20
#define L_REPEAT_MASK			0x20

#define L_DEST_ADDR_FLAG		0x80 /* Octet 5! */
#define L_TEL_LENGTH_MASK		0x0F /* Octet 5! */

#define L_PRIORITY_CLASS_SYSTEM		0x00
#define L_PRIORITY_CLASS_ALARM		0x08
#define L_PRIORITY_CLASS_HIGH		0x04
#define L_PRIORITY_CLASS_LOW		0x0C
#define L_PRIORITY_MASK			0x0C

/* Network Layer */

#define N_ROUTE_COUNT_MASK		0x70

/* Transport Layer */

#define T_BROADCAST_DATA_REQ_PDU	0x00
#define T_GROUPDATA_REQ_PDU		0x00
#define T_DATA_UNACK_REQ_PDU		0x00
#define T_DATA_REQ_PDU			0x40
#define T_CONNECT_REQ_PDU		0x80
#define T_DISCONNECT_REQ_PDU		0x81
#define T_DATA_ACK_PDU			0xC2
#define T_DATA_NAK_PDU			0xC3

/* Application Layer */
/* ACF Values for Octet 6 _and_ 7! */

#define A_GroupValue_Read		0x0000
#define A_GroupValue_Res		0x0040
#define A_GroupValue_Write		0x0080

#define A_PhysicalAddress_Write		0x00C0
#define A_PhysicalAddress_Read		0x0100
#define A_PhysicalAddress_Res		0x0140
#define A_PhysicalAddressSerialNumber_Read	0x03DC
#define A_PhysicalAddressSerialNumber_Res	0x03DD
#define A_PhysicalAddressSerialNumber_Write	0x03DE
#define A_ServiceInformation_Indication_Write	0x03DF

#define A_PropertyValue_Read		0x03D5
#define A_PropertyValue_Res		0x03D6
#define A_PropertyValue_Write		0x03D7
#define A_PropertyDescription_Read	0x03D8
#define A_PropertyDescription_Write	0x03D9

#define A_ADC_Read			0x0180
#define A_ADC_Res			0x01C0
#define A_Memory_Read			0x0200
#define A_Memory_Res			0x0240
#define A_Memory_Write			0x0280

#define A_UserMemory_Read		0x02C0
#define A_UserMemory_Res		0x02C1
#define A_UserMemory_Write		0x02C2
#define A_UserMemoryBit_Write		0x02C4
#define A_UserManufacturerInfo_Read	0x02C5
#define A_UserManufacturerInfo_Res	0x02C6

#define A_DeviceDescriptor_Read		0x0300
#define A_DeviceDescriptor_Res		0x0340
#define A_Restart			0x0380

#define A_Open_Routing_Table_Req	0x03C0
#define A_Read_Routing_Table_Req	0x03C1
#define A_Read_Routing_Table_Res	0x03C2
#define A_Write_Routing_Table_Req	0x03C3
#define A_Read_Router_Memory_Req	0x03C8
#define A_Read_Router_Memory_Res	0x03C9
#define A_Write_Router_Memory_Req	0x03CA
#define A_Read_Router_Status_Req	0x03CD
#define A_Read_Router_Status_Res	0x03CE
#define A_Write_Router_Status_Req	0x03CF

#define A_MemoryBit_Write		0x03D0
#define A_Authorize_Req			0x03D1
#define A_Authorize_Res			0x03D2
#define A_Key_Write			0x03D3
#define A_Key_Response			0x03D4

#define A_DomainAddress_Write		0x03E0
#define A_DomainAddress_Read		0x03E1
#define A_DomainAddress_Res		0x03E2
#define A_DomainAddressSelective_Read	0x03E3

#endif /* __TPLIB */
